@@ -23,6 +23,7 @@
 
 #include <winpr/crt.h>
 #include <winpr/assert.h>
+#include <winpr/cast.h>
 
 #include <freerdp/freerdp.h>
 #include <winpr/stream.h>
@@ -34,12 +35,12 @@
 
 #define TAG FREERDP_TAG("cache.glyph")
 
-static rdpGlyph* glyph_cache_get(rdpGlyphCache* glyph_cache, UINT32 id, UINT32 index);
-static BOOL glyph_cache_put(rdpGlyphCache* glyph_cache, UINT32 id, UINT32 index, rdpGlyph* entry);
+static rdpGlyph* glyph_cache_get(rdpGlyphCache* glyphCache, UINT32 id, UINT32 index);
+static BOOL glyph_cache_put(rdpGlyphCache* glyphCache, UINT32 id, UINT32 index, rdpGlyph* glyph);
 
-static const void* glyph_cache_fragment_get(rdpGlyphCache* glyph, UINT32 index, UINT32* count);
-static BOOL glyph_cache_fragment_put(rdpGlyphCache* glyph, UINT32 index, UINT32 count,
-                                     const void* entry);
+static const void* glyph_cache_fragment_get(rdpGlyphCache* glyphCache, UINT32 index, UINT32* size);
+static BOOL glyph_cache_fragment_put(rdpGlyphCache* glyphCache, UINT32 index, UINT32 size,
+                                     const void* fragment);
 
 static UINT32 update_glyph_offset(const BYTE* data, size_t length, UINT32 index, INT32* x, INT32* y,
                                   UINT32 ulCharInc, UINT32 flAccel)
@@ -62,36 +63,34 @@ static UINT32 update_glyph_offset(const BYTE* data, size_t length, UINT32 index,
 		}
 
 		if (flAccel & SO_VERTICAL)
-			*y += offset;
+			*y += WINPR_ASSERTING_INT_CAST(int32_t, offset);
 
 		if (flAccel & SO_HORIZONTAL)
-			*x += offset;
+			*x += WINPR_ASSERTING_INT_CAST(int32_t, offset);
 	}
 
 	return index;
 }
 
 static BOOL update_process_glyph(rdpContext* context, const BYTE* data, UINT32 cacheIndex, INT32* x,
-                                 INT32* y, UINT32 cacheId, UINT32 flAccel, BOOL fOpRedundant,
+                                 const INT32* y, UINT32 cacheId, UINT32 flAccel, BOOL fOpRedundant,
                                  const RDP_RECT* bound)
 {
-	INT32 sx = 0, sy = 0;
-	INT32 dx, dy;
-	rdpGlyph* glyph;
-	rdpGlyphCache* glyph_cache;
+	INT32 sx = 0;
+	INT32 sy = 0;
 
 	if (!context || !data || !x || !y || !context->graphics || !context->cache ||
 	    !context->cache->glyph)
 		return FALSE;
 
-	glyph_cache = context->cache->glyph;
-	glyph = glyph_cache_get(glyph_cache, cacheId, cacheIndex);
+	rdpGlyphCache* glyph_cache = context->cache->glyph;
+	rdpGlyph* glyph = glyph_cache_get(glyph_cache, cacheId, cacheIndex);
 
 	if (!glyph)
 		return FALSE;
 
-	dx = glyph->x + *x;
-	dy = glyph->y + *y;
+	INT32 dx = glyph->x + *x;
+	INT32 dy = glyph->y + *y;
 
 	if (dx < bound->x)
 	{
@@ -107,8 +106,8 @@ static BOOL update_process_glyph(rdpContext* context, const BYTE* data, UINT32 c
 
 	if ((dx <= (bound->x + bound->width)) && (dy <= (bound->y + bound->height)))
 	{
-		INT32 dw = glyph->cx - sx;
-		INT32 dh = glyph->cy - sy;
+		INT32 dw = WINPR_ASSERTING_INT_CAST(int32_t, glyph->cx) - sx;
+		INT32 dh = WINPR_ASSERTING_INT_CAST(int32_t, glyph->cy) - sy;
 
 		if ((dw + dx) > (bound->x + bound->width))
 			dw = (bound->x + bound->width) - (dw + dx);
@@ -124,7 +123,7 @@ static BOOL update_process_glyph(rdpContext* context, const BYTE* data, UINT32 c
 	}
 
 	if (flAccel & SO_CHAR_INC_EQUAL_BM_BASE)
-		*x += glyph->cx;
+		*x += WINPR_ASSERTING_INT_CAST(int32_t, glyph->cx);
 
 	return TRUE;
 }
@@ -136,157 +135,172 @@ static BOOL update_process_glyph_fragments(rdpContext* context, const BYTE* data
                                            INT32 opX, INT32 opY, INT32 opWidth, INT32 opHeight,
                                            BOOL fOpRedundant)
 {
-	UINT32 n;
-	UINT32 id;
-	UINT32 size;
+	UINT32 id = 0;
+	UINT32 size = 0;
 	UINT32 index = 0;
-	const BYTE* fragments;
-	rdpGraphics* graphics;
-	rdpGlyphCache* glyph_cache;
-	rdpGlyph* glyph;
-	RDP_RECT bound;
+	const BYTE* fragments = NULL;
+	RDP_RECT bound = { 0 };
+	BOOL rc = FALSE;
 
 	if (!context || !data || !context->graphics || !context->cache || !context->cache->glyph)
 		return FALSE;
 
-	graphics = context->graphics;
-	glyph_cache = context->cache->glyph;
-	glyph = graphics->Glyph_Prototype;
+	rdpGraphics* graphics = context->graphics;
+	WINPR_ASSERT(graphics);
 
-	if (!glyph)
-		return FALSE;
+	WINPR_ASSERT(context->cache);
+	rdpGlyphCache* glyph_cache = context->cache->glyph;
+	WINPR_ASSERT(glyph_cache);
 
-	/* Limit op rectangle to visible screen. */
-	if (opX < 0)
 	{
-		opWidth += opX;
-		opX = 0;
-	}
+		rdpGlyph* glyph = graphics->Glyph_Prototype;
+		if (!glyph)
+			goto fail;
 
-	if (opY < 0)
-	{
-		opHeight += opY;
-		opY = 0;
-	}
-
-	if (opWidth < 0)
-		opWidth = 0;
-
-	if (opHeight < 0)
-		opHeight = 0;
-
-	/* Limit bk rectangle to visible screen. */
-	if (bkX < 0)
-	{
-		bkWidth += bkX;
-		bkX = 0;
-	}
-
-	if (bkY < 0)
-	{
-		bkHeight += bkY;
-		bkY = 0;
-	}
-
-	if (bkWidth < 0)
-		bkWidth = 0;
-
-	if (bkHeight < 0)
-		bkHeight = 0;
-
-	if (opX + opWidth > (INT64)freerdp_settings_get_uint32(context->settings, FreeRDP_DesktopWidth))
-	{
-		/**
-		 * Some Microsoft servers send erroneous high values close to the
-		 * sint16 maximum in the OpRight field of the GlyphIndex, FastIndex and
-		 * FastGlyph drawing orders, probably a result of applications trying to
-		 * clear the text line to the very right end.
-		 * One example where this can be seen is typing in notepad.exe within
-		 * a RDP session to Windows XP Professional SP3.
-		 * This workaround prevents resulting problems in the UI callbacks.
-		 */
-		opWidth = freerdp_settings_get_uint32(context->settings, FreeRDP_DesktopWidth) - opX;
-	}
-
-	if (bkX + bkWidth > (INT64)freerdp_settings_get_uint32(context->settings, FreeRDP_DesktopWidth))
-	{
-		/**
-		 * Some Microsoft servers send erroneous high values close to the
-		 * sint16 maximum in the OpRight field of the GlyphIndex, FastIndex and
-		 * FastGlyph drawing orders, probably a result of applications trying to
-		 * clear the text line to the very right end.
-		 * One example where this can be seen is typing in notepad.exe within
-		 * a RDP session to Windows XP Professional SP3.
-		 * This workaround prevents resulting problems in the UI callbacks.
-		 */
-		bkWidth = freerdp_settings_get_uint32(context->settings, FreeRDP_DesktopWidth) - bkX;
-	}
-
-	bound.x = bkX;
-	bound.y = bkY;
-	bound.width = bkWidth;
-	bound.height = bkHeight;
-
-	if (!glyph->BeginDraw(context, opX, opY, opWidth, opHeight, bgcolor, fgcolor, fOpRedundant))
-		return FALSE;
-
-	if (!IFCALLRESULT(TRUE, glyph->SetBounds, context, bkX, bkY, bkWidth, bkHeight))
-		return FALSE;
-
-	while (index < length)
-	{
-		const UINT32 op = data[index++];
-
-		switch (op)
+		/* Limit op rectangle to visible screen. */
+		if (opX < 0)
 		{
-			case GLYPH_FRAGMENT_USE:
-				if (index + 1 >= length)
-					return FALSE;
-
-				id = data[index++];
-				fragments = (const BYTE*)glyph_cache_fragment_get(glyph_cache, id, &size);
-
-				if (fragments == NULL)
-					return FALSE;
-
-				for (n = 0; n < size;)
-				{
-					const UINT32 fop = fragments[n++];
-					n = update_glyph_offset(fragments, size, n, &x, &y, ulCharInc, flAccel);
-
-					if (!update_process_glyph(context, fragments, fop, &x, &y, cacheId, flAccel,
-					                          fOpRedundant, &bound))
-						return FALSE;
-				}
-
-				break;
-
-			case GLYPH_FRAGMENT_ADD:
-				if (index + 2 > length)
-					return FALSE;
-
-				id = data[index++];
-				size = data[index++];
-				glyph_cache_fragment_put(glyph_cache, id, size, data);
-				break;
-
-			default:
-				index = update_glyph_offset(data, length, index, &x, &y, ulCharInc, flAccel);
-
-				if (!update_process_glyph(context, data, op, &x, &y, cacheId, flAccel, fOpRedundant,
-				                          &bound))
-					return FALSE;
-
-				break;
+			opWidth += opX;
+			opX = 0;
 		}
+
+		if (opY < 0)
+		{
+			opHeight += opY;
+			opY = 0;
+		}
+
+		if (opWidth < 0)
+			opWidth = 0;
+
+		if (opHeight < 0)
+			opHeight = 0;
+
+		/* Limit bk rectangle to visible screen. */
+		if (bkX < 0)
+		{
+			bkWidth += bkX;
+			bkX = 0;
+		}
+
+		if (bkY < 0)
+		{
+			bkHeight += bkY;
+			bkY = 0;
+		}
+
+		if (bkWidth < 0)
+			bkWidth = 0;
+
+		if (bkHeight < 0)
+			bkHeight = 0;
+
+		{
+			const UINT32 w = freerdp_settings_get_uint32(context->settings, FreeRDP_DesktopWidth);
+			if (opX + opWidth > (INT64)w)
+			{
+				/**
+				 * Some Microsoft servers send erroneous high values close to the
+				 * sint16 maximum in the OpRight field of the GlyphIndex, FastIndex and
+				 * FastGlyph drawing orders, probably a result of applications trying to
+				 * clear the text line to the very right end.
+				 * One example where this can be seen is typing in notepad.exe within
+				 * a RDP session to Windows XP Professional SP3.
+				 * This workaround prevents resulting problems in the UI callbacks.
+				 */
+				opWidth = WINPR_ASSERTING_INT_CAST(int, w) - opX;
+			}
+
+			if (bkX + bkWidth > (INT64)w)
+			{
+				/**
+				 * Some Microsoft servers send erroneous high values close to the
+				 * sint16 maximum in the OpRight field of the GlyphIndex, FastIndex and
+				 * FastGlyph drawing orders, probably a result of applications trying to
+				 * clear the text line to the very right end.
+				 * One example where this can be seen is typing in notepad.exe within
+				 * a RDP session to Windows XP Professional SP3.
+				 * This workaround prevents resulting problems in the UI callbacks.
+				 */
+				bkWidth = WINPR_ASSERTING_INT_CAST(int, w) - bkX;
+			}
+		}
+
+		bound.x = WINPR_ASSERTING_INT_CAST(INT16, bkX);
+		bound.y = WINPR_ASSERTING_INT_CAST(INT16, bkY);
+		bound.width = WINPR_ASSERTING_INT_CAST(INT16, bkWidth);
+		bound.height = WINPR_ASSERTING_INT_CAST(INT16, bkHeight);
+
+		if (!glyph->BeginDraw(context, opX, opY, opWidth, opHeight, bgcolor, fgcolor, fOpRedundant))
+			goto fail;
+
+		if (!IFCALLRESULT(TRUE, glyph->SetBounds, context, bkX, bkY, bkWidth, bkHeight))
+			goto fail;
+
+		while (index < length)
+		{
+			const UINT32 op = data[index++];
+
+			switch (op)
+			{
+				case GLYPH_FRAGMENT_USE:
+					if (index + 1 > length)
+						goto fail;
+
+					id = data[index++];
+					fragments = (const BYTE*)glyph_cache_fragment_get(glyph_cache, id, &size);
+
+					if (fragments == NULL)
+						goto fail;
+
+					for (UINT32 n = 0; n < size;)
+					{
+						const UINT32 fop = fragments[n++];
+						n = update_glyph_offset(fragments, size, n, &x, &y, ulCharInc, flAccel);
+
+						if (!update_process_glyph(context, fragments, fop, &x, &y, cacheId, flAccel,
+						                          fOpRedundant, &bound))
+							goto fail;
+					}
+
+					break;
+
+				case GLYPH_FRAGMENT_ADD:
+					if (index + 2 > length)
+						goto fail;
+
+					id = data[index++];
+					size = data[index++];
+					glyph_cache_fragment_put(glyph_cache, id, size, data);
+					break;
+
+				default:
+					index = update_glyph_offset(data, length, index, &x, &y, ulCharInc, flAccel);
+
+					if (!update_process_glyph(context, data, op, &x, &y, cacheId, flAccel,
+					                          fOpRedundant, &bound))
+						goto fail;
+
+					break;
+			}
+		}
+
+		if (!glyph->EndDraw(context, opX, opY, opWidth, opHeight, bgcolor, fgcolor))
+			goto fail;
 	}
 
-	return glyph->EndDraw(context, opX, opY, opWidth, opHeight, bgcolor, fgcolor);
+	rc = TRUE;
+
+fail:
+	return rc;
 }
 
 static BOOL update_gdi_glyph_index(rdpContext* context, GLYPH_INDEX_ORDER* glyphIndex)
 {
-	INT32 bkWidth = 0, bkHeight = 0, opWidth = 0, opHeight = 0;
+	INT32 bkWidth = 0;
+	INT32 bkHeight = 0;
+	INT32 opWidth = 0;
+	INT32 opHeight = 0;
 
 	if (!context || !glyphIndex || !context->cache)
 		return FALSE;
@@ -307,26 +321,27 @@ static BOOL update_gdi_glyph_index(rdpContext* context, GLYPH_INDEX_ORDER* glyph
 	    context, glyphIndex->data, glyphIndex->cbData, glyphIndex->cacheId, glyphIndex->ulCharInc,
 	    glyphIndex->flAccel, glyphIndex->backColor, glyphIndex->foreColor, glyphIndex->x,
 	    glyphIndex->y, glyphIndex->bkLeft, glyphIndex->bkTop, bkWidth, bkHeight, glyphIndex->opLeft,
-	    glyphIndex->opTop, opWidth, opHeight, glyphIndex->fOpRedundant);
+	    glyphIndex->opTop, opWidth, opHeight,
+	    WINPR_ASSERTING_INT_CAST(int32_t, glyphIndex->fOpRedundant));
 }
 
 static BOOL update_gdi_fast_index(rdpContext* context, const FAST_INDEX_ORDER* fastIndex)
 {
-	INT32 x, y;
-	INT32 opLeft, opTop;
-	INT32 opRight, opBottom;
-	INT32 opWidth = 0, opHeight = 0;
-	INT32 bkWidth = 0, bkHeight = 0;
+	INT32 opWidth = 0;
+	INT32 opHeight = 0;
+	INT32 bkWidth = 0;
+	INT32 bkHeight = 0;
+	BOOL rc = FALSE;
 
 	if (!context || !fastIndex || !context->cache)
 		return FALSE;
 
-	opLeft = fastIndex->opLeft;
-	opTop = fastIndex->opTop;
-	opRight = fastIndex->opRight;
-	opBottom = fastIndex->opBottom;
-	x = fastIndex->x;
-	y = fastIndex->y;
+	INT32 opLeft = fastIndex->opLeft;
+	INT32 opTop = fastIndex->opTop;
+	INT32 opRight = fastIndex->opRight;
+	INT32 opBottom = fastIndex->opBottom;
+	INT32 x = fastIndex->x;
+	INT32 y = fastIndex->y;
 
 	if (opBottom == -32768)
 	{
@@ -375,21 +390,31 @@ static BOOL update_gdi_fast_index(rdpContext* context, const FAST_INDEX_ORDER* f
 	if (opBottom > opTop)
 		opHeight = opBottom - opTop + 1;
 
-	return update_process_glyph_fragments(
-	    context, fastIndex->data, fastIndex->cbData, fastIndex->cacheId, fastIndex->ulCharInc,
-	    fastIndex->flAccel, fastIndex->backColor, fastIndex->foreColor, x, y, fastIndex->bkLeft,
-	    fastIndex->bkTop, bkWidth, bkHeight, opLeft, opTop, opWidth, opHeight, FALSE);
+	if (!update_process_glyph_fragments(
+	        context, fastIndex->data, fastIndex->cbData, fastIndex->cacheId, fastIndex->ulCharInc,
+	        fastIndex->flAccel, fastIndex->backColor, fastIndex->foreColor, x, y, fastIndex->bkLeft,
+	        fastIndex->bkTop, bkWidth, bkHeight, opLeft, opTop, opWidth, opHeight, FALSE))
+		goto fail;
+
+	rc = TRUE;
+fail:
+	return rc;
 }
 
 static BOOL update_gdi_fast_glyph(rdpContext* context, const FAST_GLYPH_ORDER* fastGlyph)
 {
-	INT32 x, y;
+	INT32 x = 0;
+	INT32 y = 0;
 	BYTE text_data[4] = { 0 };
-	INT32 opLeft, opTop;
-	INT32 opRight, opBottom;
-	INT32 opWidth = 0, opHeight = 0;
-	INT32 bkWidth = 0, bkHeight = 0;
-	rdpCache* cache;
+	INT32 opLeft = 0;
+	INT32 opTop = 0;
+	INT32 opRight = 0;
+	INT32 opBottom = 0;
+	INT32 opWidth = 0;
+	INT32 opHeight = 0;
+	INT32 bkWidth = 0;
+	INT32 bkHeight = 0;
+	rdpCache* cache = NULL;
 
 	if (!context || !fastGlyph || !context->cache)
 		return FALSE;
@@ -438,7 +463,7 @@ static BOOL update_gdi_fast_glyph(rdpContext* context, const FAST_GLYPH_ORDER* f
 	if ((fastGlyph->cbData > 1) && (fastGlyph->glyphData.aj))
 	{
 		/* got option font that needs to go into cache */
-		rdpGlyph* glyph;
+		rdpGlyph* glyph = NULL;
 		const GLYPH_DATA_V2* glyphData = &fastGlyph->glyphData;
 
 		glyph = Glyph_Alloc(context, glyphData->x, glyphData->y, glyphData->cx, glyphData->cy,
@@ -477,24 +502,17 @@ static BOOL update_gdi_fast_glyph(rdpContext* context, const FAST_GLYPH_ORDER* f
 
 static BOOL update_gdi_cache_glyph(rdpContext* context, const CACHE_GLYPH_ORDER* cacheGlyph)
 {
-	UINT32 i;
-	rdpCache* cache;
-
 	if (!context || !cacheGlyph || !context->cache)
 		return FALSE;
 
-	cache = context->cache;
+	rdpCache* cache = context->cache;
 
-	for (i = 0; i < cacheGlyph->cGlyphs; i++)
+	for (size_t i = 0; i < cacheGlyph->cGlyphs; i++)
 	{
 		const GLYPH_DATA* glyph_data = &cacheGlyph->glyphData[i];
-		rdpGlyph* glyph;
-
-		if (!glyph_data)
-			return FALSE;
-
-		if (!(glyph = Glyph_Alloc(context, glyph_data->x, glyph_data->y, glyph_data->cx,
-		                          glyph_data->cy, glyph_data->cb, glyph_data->aj)))
+		rdpGlyph* glyph = Glyph_Alloc(context, glyph_data->x, glyph_data->y, glyph_data->cx,
+		                              glyph_data->cy, glyph_data->cb, glyph_data->aj);
+		if (!glyph)
 			return FALSE;
 
 		if (!glyph_cache_put(cache->glyph, cacheGlyph->cacheId, glyph_data->cacheIndex, glyph))
@@ -509,24 +527,16 @@ static BOOL update_gdi_cache_glyph(rdpContext* context, const CACHE_GLYPH_ORDER*
 
 static BOOL update_gdi_cache_glyph_v2(rdpContext* context, const CACHE_GLYPH_V2_ORDER* cacheGlyphV2)
 {
-	UINT32 i;
-	rdpCache* cache;
-
 	if (!context || !cacheGlyphV2 || !context->cache)
 		return FALSE;
 
-	cache = context->cache;
+	rdpCache* cache = context->cache;
 
-	for (i = 0; i < cacheGlyphV2->cGlyphs; i++)
+	for (size_t i = 0; i < cacheGlyphV2->cGlyphs; i++)
 	{
 		const GLYPH_DATA_V2* glyphData = &cacheGlyphV2->glyphData[i];
-		rdpGlyph* glyph;
-
-		if (!glyphData)
-			return FALSE;
-
-		glyph = Glyph_Alloc(context, glyphData->x, glyphData->y, glyphData->cx, glyphData->cy,
-		                    glyphData->cb, glyphData->aj);
+		rdpGlyph* glyph = Glyph_Alloc(context, glyphData->x, glyphData->y, glyphData->cx,
+		                              glyphData->cy, glyphData->cb, glyphData->aj);
 
 		if (!glyph)
 			return FALSE;
@@ -543,28 +553,25 @@ static BOOL update_gdi_cache_glyph_v2(rdpContext* context, const CACHE_GLYPH_V2_
 
 rdpGlyph* glyph_cache_get(rdpGlyphCache* glyphCache, UINT32 id, UINT32 index)
 {
-	rdpGlyph* glyph;
-
 	WINPR_ASSERT(glyphCache);
 
 	WLog_Print(glyphCache->log, WLOG_DEBUG, "GlyphCacheGet: id: %" PRIu32 " index: %" PRIu32 "", id,
 	           index);
 
-	if (id > 9)
+	if (id >= ARRAYSIZE(glyphCache->glyphCache))
 	{
 		WLog_ERR(TAG, "invalid glyph cache id: %" PRIu32 "", id);
 		return NULL;
 	}
 
-	WINPR_ASSERT(glyphCache->glyphCache);
-	if (index > glyphCache->glyphCache[id].number)
+	GLYPH_CACHE* cache = &glyphCache->glyphCache[id];
+	if (index > cache->number)
 	{
 		WLog_ERR(TAG, "index %" PRIu32 " out of range for cache id: %" PRIu32 "", index, id);
 		return NULL;
 	}
 
-	glyph = glyphCache->glyphCache[id].entries[index];
-
+	rdpGlyph* glyph = cache->entries[index];
 	if (!glyph)
 		WLog_ERR(TAG, "no glyph found at cache index: %" PRIu32 " in cache id: %" PRIu32 "", index,
 		         id);
@@ -574,18 +581,16 @@ rdpGlyph* glyph_cache_get(rdpGlyphCache* glyphCache, UINT32 id, UINT32 index)
 
 BOOL glyph_cache_put(rdpGlyphCache* glyphCache, UINT32 id, UINT32 index, rdpGlyph* glyph)
 {
-	rdpGlyph* prevGlyph;
-
 	WINPR_ASSERT(glyphCache);
 
-	if (id > 9)
+	if (id >= ARRAYSIZE(glyphCache->glyphCache))
 	{
 		WLog_ERR(TAG, "invalid glyph cache id: %" PRIu32 "", id);
 		return FALSE;
 	}
 
-	WINPR_ASSERT(glyphCache->glyphCache);
-	if (index >= glyphCache->glyphCache[id].number)
+	GLYPH_CACHE* cache = &glyphCache->glyphCache[id];
+	if (index >= cache->number)
 	{
 		WLog_ERR(TAG, "invalid glyph cache index: %" PRIu32 " in cache id: %" PRIu32 "", index, id);
 		return FALSE;
@@ -593,7 +598,7 @@ BOOL glyph_cache_put(rdpGlyphCache* glyphCache, UINT32 id, UINT32 index, rdpGlyp
 
 	WLog_Print(glyphCache->log, WLOG_DEBUG, "GlyphCachePut: id: %" PRIu32 " index: %" PRIu32 "", id,
 	           index);
-	prevGlyph = glyphCache->glyphCache[id].entries[index];
+	rdpGlyph* prevGlyph = cache->entries[index];
 
 	if (prevGlyph)
 	{
@@ -601,13 +606,13 @@ BOOL glyph_cache_put(rdpGlyphCache* glyphCache, UINT32 id, UINT32 index, rdpGlyp
 		prevGlyph->Free(glyphCache->context, prevGlyph);
 	}
 
-	glyphCache->glyphCache[id].entries[index] = glyph;
+	cache->entries[index] = glyph;
 	return TRUE;
 }
 
 const void* glyph_cache_fragment_get(rdpGlyphCache* glyphCache, UINT32 index, UINT32* size)
 {
-	void* fragment;
+	void* fragment = NULL;
 
 	WINPR_ASSERT(glyphCache);
 	WINPR_ASSERT(glyphCache->fragCache.entries);
@@ -632,9 +637,6 @@ const void* glyph_cache_fragment_get(rdpGlyphCache* glyphCache, UINT32 index, UI
 BOOL glyph_cache_fragment_put(rdpGlyphCache* glyphCache, UINT32 index, UINT32 size,
                               const void* fragment)
 {
-	void* prevFragment;
-	void* copy;
-
 	WINPR_ASSERT(glyphCache);
 	WINPR_ASSERT(glyphCache->fragCache.entries);
 
@@ -644,7 +646,10 @@ BOOL glyph_cache_fragment_put(rdpGlyphCache* glyphCache, UINT32 index, UINT32 si
 		return FALSE;
 	}
 
-	copy = malloc(size);
+	if (size == 0)
+		return FALSE;
+
+	void* copy = malloc(size);
 
 	if (!copy)
 		return FALSE;
@@ -652,7 +657,8 @@ BOOL glyph_cache_fragment_put(rdpGlyphCache* glyphCache, UINT32 index, UINT32 si
 	WLog_Print(glyphCache->log, WLOG_DEBUG,
 	           "GlyphCacheFragmentPut: index: %" PRIu32 " size: %" PRIu32 "", index, size);
 	CopyMemory(copy, fragment, size);
-	prevFragment = glyphCache->fragCache.entries[index].fragment;
+
+	void* prevFragment = glyphCache->fragCache.entries[index].fragment;
 	glyphCache->fragCache.entries[index].fragment = copy;
 	glyphCache->fragCache.entries[index].size = size;
 	free(prevFragment);
@@ -678,9 +684,8 @@ void glyph_cache_register_callbacks(rdpUpdate* update)
 
 rdpGlyphCache* glyph_cache_new(rdpContext* context)
 {
-	size_t i;
-	rdpGlyphCache* glyphCache;
-	rdpSettings* settings;
+	rdpGlyphCache* glyphCache = NULL;
+	rdpSettings* settings = NULL;
 
 	WINPR_ASSERT(context);
 
@@ -695,7 +700,7 @@ rdpGlyphCache* glyph_cache_new(rdpContext* context)
 	glyphCache->log = WLog_Get("com.freerdp.cache.glyph");
 	glyphCache->context = context;
 
-	for (i = 0; i < 10; i++)
+	for (size_t i = 0; i < 10; i++)
 	{
 		const GLYPH_CACHE_DEFINITION* currentGlyph =
 		    freerdp_settings_get_pointer_array(settings, FreeRDP_GlyphCache, i);
@@ -710,7 +715,10 @@ rdpGlyphCache* glyph_cache_new(rdpContext* context)
 
 	return glyphCache;
 fail:
+	WINPR_PRAGMA_DIAG_PUSH
+	WINPR_PRAGMA_DIAG_IGNORED_MISMATCHED_DEALLOC
 	glyph_cache_free(glyphCache);
+	WINPR_PRAGMA_DIAG_POP
 	return NULL;
 }
 
@@ -718,18 +726,16 @@ void glyph_cache_free(rdpGlyphCache* glyphCache)
 {
 	if (glyphCache)
 	{
-		size_t i;
 		GLYPH_CACHE* cache = glyphCache->glyphCache;
 
-		for (i = 0; i < 10; i++)
+		for (size_t i = 0; i < 10; i++)
 		{
-			UINT32 j;
 			rdpGlyph** entries = cache[i].entries;
 
 			if (!entries)
 				continue;
 
-			for (j = 0; j < cache[i].number; j++)
+			for (size_t j = 0; j < cache[i].number; j++)
 			{
 				rdpGlyph* glyph = entries[j];
 
@@ -740,11 +746,11 @@ void glyph_cache_free(rdpGlyphCache* glyphCache)
 				}
 			}
 
-			free(entries);
+			free((void*)entries);
 			cache[i].entries = NULL;
 		}
 
-		for (i = 0; i < ARRAYSIZE(glyphCache->fragCache.entries); i++)
+		for (size_t i = 0; i < ARRAYSIZE(glyphCache->fragCache.entries); i++)
 		{
 			free(glyphCache->fragCache.entries[i].fragment);
 			glyphCache->fragCache.entries[i].fragment = NULL;
@@ -756,8 +762,7 @@ void glyph_cache_free(rdpGlyphCache* glyphCache)
 
 CACHE_GLYPH_ORDER* copy_cache_glyph_order(rdpContext* context, const CACHE_GLYPH_ORDER* glyph)
 {
-	size_t x;
-	CACHE_GLYPH_ORDER* dst;
+	CACHE_GLYPH_ORDER* dst = NULL;
 
 	WINPR_ASSERT(context);
 
@@ -768,7 +773,7 @@ CACHE_GLYPH_ORDER* copy_cache_glyph_order(rdpContext* context, const CACHE_GLYPH
 
 	*dst = *glyph;
 
-	for (x = 0; x < glyph->cGlyphs; x++)
+	for (size_t x = 0; x < glyph->cGlyphs; x++)
 	{
 		const GLYPH_DATA* src = &glyph->glyphData[x];
 		GLYPH_DATA* data = &dst->glyphData[x];
@@ -804,13 +809,11 @@ fail:
 	return NULL;
 }
 
-void free_cache_glyph_order(rdpContext* context, CACHE_GLYPH_ORDER* glyph)
+void free_cache_glyph_order(WINPR_ATTR_UNUSED rdpContext* context, CACHE_GLYPH_ORDER* glyph)
 {
 	if (glyph)
 	{
-		size_t x;
-
-		for (x = 0; x < ARRAYSIZE(glyph->glyphData); x++)
+		for (size_t x = 0; x < ARRAYSIZE(glyph->glyphData); x++)
 			free(glyph->glyphData[x].aj);
 
 		free(glyph->unicodeCharacters);
@@ -822,8 +825,7 @@ void free_cache_glyph_order(rdpContext* context, CACHE_GLYPH_ORDER* glyph)
 CACHE_GLYPH_V2_ORDER* copy_cache_glyph_v2_order(rdpContext* context,
                                                 const CACHE_GLYPH_V2_ORDER* glyph)
 {
-	size_t x;
-	CACHE_GLYPH_V2_ORDER* dst;
+	CACHE_GLYPH_V2_ORDER* dst = NULL;
 
 	WINPR_ASSERT(context);
 
@@ -834,7 +836,7 @@ CACHE_GLYPH_V2_ORDER* copy_cache_glyph_v2_order(rdpContext* context,
 
 	*dst = *glyph;
 
-	for (x = 0; x < glyph->cGlyphs; x++)
+	for (size_t x = 0; x < glyph->cGlyphs; x++)
 	{
 		const GLYPH_DATA_V2* src = &glyph->glyphData[x];
 		GLYPH_DATA_V2* data = &dst->glyphData[x];
@@ -870,13 +872,11 @@ fail:
 	return NULL;
 }
 
-void free_cache_glyph_v2_order(rdpContext* context, CACHE_GLYPH_V2_ORDER* glyph)
+void free_cache_glyph_v2_order(WINPR_ATTR_UNUSED rdpContext* context, CACHE_GLYPH_V2_ORDER* glyph)
 {
 	if (glyph)
 	{
-		size_t x;
-
-		for (x = 0; x < ARRAYSIZE(glyph->glyphData); x++)
+		for (size_t x = 0; x < ARRAYSIZE(glyph->glyphData); x++)
 			free(glyph->glyphData[x].aj);
 
 		free(glyph->unicodeCharacters);
