@@ -56,7 +56,7 @@ typedef struct
 	BOOL mode;
 } RFX_PROGRESSIVE_UPGRADE_STATE;
 
-static INLINE void
+static inline void
 progressive_component_codec_quant_read(wStream* WINPR_RESTRICT s,
                                        RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT quantVal)
 {
@@ -78,7 +78,7 @@ progressive_component_codec_quant_read(wStream* WINPR_RESTRICT s,
 	quantVal->HH1 = b >> 4;
 }
 
-static INLINE void progressive_rfx_quant_add(const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT q1,
+static inline void progressive_rfx_quant_add(const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT q1,
                                              const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT q2,
                                              RFX_COMPONENT_CODEC_QUANT* dst)
 {
@@ -94,7 +94,7 @@ static INLINE void progressive_rfx_quant_add(const RFX_COMPONENT_CODEC_QUANT* WI
 	dst->LL3 = q1->LL3 + q2->LL3; /* LL3 */
 }
 
-static INLINE void progressive_rfx_quant_lsub(RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT q, int val)
+static inline void progressive_rfx_quant_lsub(RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT q, int val)
 {
 	q->HL1 -= val; /* HL1 */
 	q->LH1 -= val; /* LH1 */
@@ -108,7 +108,7 @@ static INLINE void progressive_rfx_quant_lsub(RFX_COMPONENT_CODEC_QUANT* WINPR_R
 	q->LL3 -= val; /* LL3 */
 }
 
-static INLINE void progressive_rfx_quant_sub(const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT q1,
+static inline void progressive_rfx_quant_sub(const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT q1,
                                              const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT q2,
                                              RFX_COMPONENT_CODEC_QUANT* dst)
 {
@@ -124,7 +124,7 @@ static INLINE void progressive_rfx_quant_sub(const RFX_COMPONENT_CODEC_QUANT* WI
 	dst->LL3 = q1->LL3 - q2->LL3; /* LL3 */
 }
 
-static INLINE BOOL
+static inline BOOL
 progressive_rfx_quant_lcmp_less_equal(const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT q, int val)
 {
 	if (q->HL1 > val)
@@ -160,7 +160,7 @@ progressive_rfx_quant_lcmp_less_equal(const RFX_COMPONENT_CODEC_QUANT* WINPR_RES
 	return TRUE;
 }
 
-static INLINE BOOL
+static inline BOOL
 progressive_rfx_quant_lcmp_greater_equal(const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT q, int val)
 {
 	if (q->HL1 < val)
@@ -196,7 +196,7 @@ progressive_rfx_quant_lcmp_greater_equal(const RFX_COMPONENT_CODEC_QUANT* WINPR_
 	return TRUE;
 }
 
-static INLINE BOOL
+static inline BOOL
 progressive_rfx_quant_cmp_equal(const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT q1,
                                 const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT q2)
 {
@@ -233,7 +233,7 @@ progressive_rfx_quant_cmp_equal(const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT
 	return TRUE;
 }
 
-static INLINE BOOL progressive_set_surface_data(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
+static inline BOOL progressive_set_surface_data(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
                                                 UINT16 surfaceId,
                                                 PROGRESSIVE_SURFACE_CONTEXT* WINPR_RESTRICT pData)
 {
@@ -247,7 +247,7 @@ static INLINE BOOL progressive_set_surface_data(PROGRESSIVE_CONTEXT* WINPR_RESTR
 	return TRUE;
 }
 
-static INLINE PROGRESSIVE_SURFACE_CONTEXT*
+static inline PROGRESSIVE_SURFACE_CONTEXT*
 progressive_get_surface_data(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive, UINT16 surfaceId)
 {
 	void* key = (void*)(((ULONG_PTR)surfaceId) + 1);
@@ -290,7 +290,7 @@ static void progressive_surface_context_free(void* ptr)
 	winpr_aligned_free(surface);
 }
 
-static INLINE RFX_PROGRESSIVE_TILE* progressive_tile_new(void)
+static inline RFX_PROGRESSIVE_TILE* progressive_tile_new(void)
 {
 	RFX_PROGRESSIVE_TILE* tile = winpr_aligned_calloc(1, sizeof(RFX_PROGRESSIVE_TILE), 32);
 	if (!tile)
@@ -300,19 +300,26 @@ static INLINE RFX_PROGRESSIVE_TILE* progressive_tile_new(void)
 	tile->height = 64;
 	tile->stride = 4 * tile->width;
 
-	size_t dataLen = 1ull * tile->stride * tile->height;
-	tile->data = (BYTE*)winpr_aligned_malloc(dataLen, 16);
-	if (!tile->data)
-		goto fail;
-	memset(tile->data, 0xFF, dataLen);
+	{
+		const size_t dataLen = 1ull * tile->stride * tile->height;
+		tile->data = (BYTE*)winpr_aligned_malloc(dataLen, 16);
+		if (!tile->data)
+			goto fail;
+		memset(tile->data, 0xFF, dataLen);
+	}
+
+	{
+		const size_t signLen = (8192ULL + 32ULL) * 3ULL;
+		tile->sign = (BYTE*)winpr_aligned_malloc(signLen, 16);
+	}
 
-	size_t signLen = (8192ULL + 32ULL) * 3ULL;
-	tile->sign = (BYTE*)winpr_aligned_malloc(signLen, 16);
 	if (!tile->sign)
 		goto fail;
 
-	size_t currentLen = (8192ULL + 32ULL) * 3ULL;
-	tile->current = (BYTE*)winpr_aligned_malloc(currentLen, 16);
+	{
+		const size_t currentLen = (8192ULL + 32ULL) * 3ULL;
+		tile->current = (BYTE*)winpr_aligned_malloc(currentLen, 16);
+	}
 	if (!tile->current)
 		goto fail;
 
@@ -323,7 +330,7 @@ fail:
 	return NULL;
 }
 
-static INLINE BOOL
+static inline BOOL
 progressive_allocate_tile_cache(PROGRESSIVE_SURFACE_CONTEXT* WINPR_RESTRICT surface, size_t min)
 {
 	size_t oldIndex = 0;
@@ -387,7 +394,7 @@ static PROGRESSIVE_SURFACE_CONTEXT* progressive_surface_context_new(UINT16 surfa
 	return surface;
 }
 
-static INLINE BOOL
+static inline BOOL
 progressive_surface_tile_replace(PROGRESSIVE_SURFACE_CONTEXT* WINPR_RESTRICT surface,
                                  PROGRESSIVE_BLOCK_REGION* WINPR_RESTRICT region,
                                  const RFX_PROGRESSIVE_TILE* WINPR_RESTRICT tile, BOOL upgrade)
@@ -518,27 +525,32 @@ int progressive_delete_surface_context(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progr
  * LL3      4015        9x9         81
  */
 
-static INLINE void progressive_rfx_idwt_x(const INT16* WINPR_RESTRICT pLowBand, size_t nLowStep,
+static int16_t clampi16(int val)
+{
+	if (val < INT16_MIN)
+		return INT16_MIN;
+	if (val > INT16_MAX)
+		return INT16_MAX;
+	return (int16_t)val;
+}
+
+static inline void progressive_rfx_idwt_x(const INT16* WINPR_RESTRICT pLowBand, size_t nLowStep,
                                           const INT16* WINPR_RESTRICT pHighBand, size_t nHighStep,
                                           INT16* WINPR_RESTRICT pDstBand, size_t nDstStep,
                                           size_t nLowCount, size_t nHighCount, size_t nDstCount)
 {
-	INT16 L0 = 0;
-	INT16 H0 = 0;
 	INT16 H1 = 0;
-	INT16 X0 = 0;
 	INT16 X1 = 0;
-	INT16 X2 = 0;
 
 	for (size_t i = 0; i < nDstCount; i++)
 	{
 		const INT16* pL = pLowBand;
 		const INT16* pH = pHighBand;
 		INT16* pX = pDstBand;
-		H0 = *pH++;
-		L0 = *pL++;
-		X0 = WINPR_ASSERTING_INT_CAST(int16_t, L0 - H0);
-		X2 = WINPR_ASSERTING_INT_CAST(int16_t, L0 - H0);
+		INT16 H0 = *pH++;
+		INT16 L0 = *pL++;
+		INT16 X0 = clampi16((int32_t)L0 - H0);
+		INT16 X2 = clampi16((int32_t)L0 - H0);
 
 		for (size_t j = 0; j < (nHighCount - 1); j++)
 		{
@@ -546,8 +558,8 @@ static INLINE void progressive_rfx_idwt_x(const INT16* WINPR_RESTRICT pLowBand,
 			pH++;
 			L0 = *pL;
 			pL++;
-			X2 = WINPR_ASSERTING_INT_CAST(int16_t, L0 - ((H0 + H1) / 2));
-			X1 = WINPR_ASSERTING_INT_CAST(int16_t, ((X0 + X2) / 2) + (2 * H0));
+			X2 = clampi16((int32_t)L0 - ((H0 + H1) / 2));
+			X1 = clampi16((int32_t)((X0 + X2) / 2) + (2 * H0));
 			pX[0] = X0;
 			pX[1] = X1;
 			pX += 2;
@@ -560,15 +572,15 @@ static INLINE void progressive_rfx_idwt_x(const INT16* WINPR_RESTRICT pLowBand,
 			if (nLowCount <= nHighCount)
 			{
 				pX[0] = X2;
-				pX[1] = WINPR_ASSERTING_INT_CAST(int16_t, X2 + (2 * H0));
+				pX[1] = clampi16((int32_t)X2 + (2 * H0));
 			}
 			else
 			{
 				L0 = *pL;
 				pL++;
-				X0 = WINPR_ASSERTING_INT_CAST(int16_t, L0 - H0);
+				X0 = clampi16((int32_t)L0 - H0);
 				pX[0] = X2;
-				pX[1] = WINPR_ASSERTING_INT_CAST(int16_t, ((X0 + X2) / 2) + (2 * H0));
+				pX[1] = clampi16((int32_t)((X0 + X2) / 2) + (2 * H0));
 				pX[2] = X0;
 			}
 		}
@@ -576,13 +588,13 @@ static INLINE void progressive_rfx_idwt_x(const INT16* WINPR_RESTRICT pLowBand,
 		{
 			L0 = *pL;
 			pL++;
-			X0 = WINPR_ASSERTING_INT_CAST(int16_t, L0 - (H0 / 2));
+			X0 = clampi16((int32_t)L0 - (H0 / 2));
 			pX[0] = X2;
-			pX[1] = WINPR_ASSERTING_INT_CAST(int16_t, ((X0 + X2) / 2) + (2 * H0));
+			pX[1] = clampi16((int32_t)((X0 + X2) / 2) + (2 * H0));
 			pX[2] = X0;
 			L0 = *pL;
 			pL++;
-			pX[3] = WINPR_ASSERTING_INT_CAST(int16_t, (X0 + L0) / 2);
+			pX[3] = clampi16((int32_t)(X0 + L0) / 2);
 		}
 
 		pLowBand += nLowStep;
@@ -591,29 +603,24 @@ static INLINE void progressive_rfx_idwt_x(const INT16* WINPR_RESTRICT pLowBand,
 	}
 }
 
-static INLINE void progressive_rfx_idwt_y(const INT16* WINPR_RESTRICT pLowBand, size_t nLowStep,
+static inline void progressive_rfx_idwt_y(const INT16* WINPR_RESTRICT pLowBand, size_t nLowStep,
                                           const INT16* WINPR_RESTRICT pHighBand, size_t nHighStep,
                                           INT16* WINPR_RESTRICT pDstBand, size_t nDstStep,
                                           size_t nLowCount, size_t nHighCount, size_t nDstCount)
 {
-	INT16 L0 = 0;
-	INT16 H0 = 0;
-	INT16 H1 = 0;
-	INT16 X0 = 0;
-	INT16 X1 = 0;
-	INT16 X2 = 0;
-
 	for (size_t i = 0; i < nDstCount; i++)
 	{
+		INT16 H1 = 0;
+		INT16 X1 = 0;
 		const INT16* pL = pLowBand;
 		const INT16* pH = pHighBand;
 		INT16* pX = pDstBand;
-		H0 = *pH;
+		INT16 H0 = *pH;
 		pH += nHighStep;
-		L0 = *pL;
+		INT16 L0 = *pL;
 		pL += nLowStep;
-		X0 = WINPR_ASSERTING_INT_CAST(int16_t, L0 - H0);
-		X2 = WINPR_ASSERTING_INT_CAST(int16_t, L0 - H0);
+		int16_t X0 = clampi16((int32_t)L0 - H0);
+		int16_t X2 = clampi16((int32_t)L0 - H0);
 
 		for (size_t j = 0; j < (nHighCount - 1); j++)
 		{
@@ -621,8 +628,8 @@ static INLINE void progressive_rfx_idwt_y(const INT16* WINPR_RESTRICT pLowBand,
 			pH += nHighStep;
 			L0 = *pL;
 			pL += nLowStep;
-			X2 = WINPR_ASSERTING_INT_CAST(int16_t, L0 - ((H0 + H1) / 2));
-			X1 = WINPR_ASSERTING_INT_CAST(int16_t, ((X0 + X2) / 2) + (2 * H0));
+			X2 = clampi16((int32_t)L0 - ((H0 + H1) / 2));
+			X1 = clampi16((int32_t)((X0 + X2) / 2) + (2 * H0));
 			*pX = X0;
 			pX += nDstStep;
 			*pX = X1;
@@ -637,15 +644,15 @@ static INLINE void progressive_rfx_idwt_y(const INT16* WINPR_RESTRICT pLowBand,
 			{
 				*pX = X2;
 				pX += nDstStep;
-				*pX = WINPR_ASSERTING_INT_CAST(int16_t, X2 + (2 * H0));
+				*pX = clampi16((int32_t)X2 + (2 * H0));
 			}
 			else
 			{
 				L0 = *pL;
-				X0 = WINPR_ASSERTING_INT_CAST(int16_t, L0 - H0);
+				X0 = clampi16((int32_t)L0 - H0);
 				*pX = X2;
 				pX += nDstStep;
-				*pX = WINPR_ASSERTING_INT_CAST(int16_t, ((X0 + X2) / 2) + (2 * H0));
+				*pX = clampi16((int32_t)((X0 + X2) / 2) + (2 * H0));
 				pX += nDstStep;
 				*pX = X0;
 			}
@@ -654,15 +661,15 @@ static INLINE void progressive_rfx_idwt_y(const INT16* WINPR_RESTRICT pLowBand,
 		{
 			L0 = *pL;
 			pL += nLowStep;
-			X0 = WINPR_ASSERTING_INT_CAST(int16_t, L0 - (H0 / 2));
+			X0 = clampi16((int32_t)L0 - (H0 / 2));
 			*pX = X2;
 			pX += nDstStep;
-			*pX = WINPR_ASSERTING_INT_CAST(int16_t, ((X0 + X2) / 2) + (2 * H0));
+			*pX = clampi16((int32_t)((X0 + X2) / 2) + (2 * H0));
 			pX += nDstStep;
 			*pX = X0;
 			pX += nDstStep;
 			L0 = *pL;
-			*pX = WINPR_ASSERTING_INT_CAST(int16_t, (X0 + L0) / 2);
+			*pX = clampi16((int32_t)(X0 + L0) / 2);
 		}
 
 		pLowBand++;
@@ -671,12 +678,12 @@ static INLINE void progressive_rfx_idwt_y(const INT16* WINPR_RESTRICT pLowBand,
 	}
 }
 
-static INLINE size_t progressive_rfx_get_band_l_count(size_t level)
+static inline size_t progressive_rfx_get_band_l_count(size_t level)
 {
 	return (64 >> level) + 1;
 }
 
-static INLINE size_t progressive_rfx_get_band_h_count(size_t level)
+static inline size_t progressive_rfx_get_band_h_count(size_t level)
 {
 	if (level == 1)
 		return (64 >> 1) - 1;
@@ -684,7 +691,7 @@ static INLINE size_t progressive_rfx_get_band_h_count(size_t level)
 		return (64 + (1 << (level - 1))) >> level;
 }
 
-static INLINE void progressive_rfx_dwt_2d_decode_block(INT16* WINPR_RESTRICT buffer,
+static inline void progressive_rfx_dwt_2d_decode_block(INT16* WINPR_RESTRICT buffer,
                                                        INT16* WINPR_RESTRICT temp, size_t level)
 {
 	size_t nDstStepX = 0;
@@ -736,7 +743,7 @@ void rfx_dwt_2d_extrapolate_decode(INT16* WINPR_RESTRICT buffer, INT16* WINPR_RE
 	progressive_rfx_dwt_2d_decode_block(&buffer[0], temp, 1);
 }
 
-static INLINE int progressive_rfx_dwt_2d_decode(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
+static inline int progressive_rfx_dwt_2d_decode(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
                                                 INT16* WINPR_RESTRICT buffer,
                                                 INT16* WINPR_RESTRICT current, BOOL coeffDiff,
                                                 BOOL extrapolate, BOOL reverse)
@@ -746,8 +753,8 @@ static INLINE int progressive_rfx_dwt_2d_decode(PROGRESSIVE_CONTEXT* WINPR_RESTR
 	if (!progressive || !buffer || !current)
 		return -1;
 
-	const size_t belements = 4096;
-	const size_t bsize = belements * sizeof(INT16);
+	const uint32_t belements = 4096;
+	const uint32_t bsize = belements * sizeof(INT16);
 	if (reverse)
 		memcpy(buffer, current, bsize);
 	else if (!coeffDiff)
@@ -773,7 +780,7 @@ static INLINE int progressive_rfx_dwt_2d_decode(PROGRESSIVE_CONTEXT* WINPR_RESTR
 	return 1;
 }
 
-static INLINE void progressive_rfx_decode_block(const primitives_t* prims,
+static inline void progressive_rfx_decode_block(const primitives_t* prims,
                                                 INT16* WINPR_RESTRICT buffer, UINT32 length,
                                                 UINT32 shift)
 {
@@ -783,7 +790,7 @@ static INLINE void progressive_rfx_decode_block(const primitives_t* prims,
 	prims->lShiftC_16s_inplace(buffer, shift, length);
 }
 
-static INLINE int
+static inline int
 progressive_rfx_decode_component(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
                                  const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT shift,
                                  const BYTE* WINPR_RESTRICT data, UINT32 length,
@@ -832,7 +839,7 @@ progressive_rfx_decode_component(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive
 	                                     FALSE);
 }
 
-static INLINE int
+static inline int
 progressive_decompress_tile_first(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
                                   RFX_PROGRESSIVE_TILE* WINPR_RESTRICT tile,
                                   PROGRESSIVE_BLOCK_REGION* WINPR_RESTRICT region,
@@ -965,15 +972,17 @@ progressive_decompress_tile_first(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressiv
 	if (rc < 0)
 		goto fail;
 
-	const INT16** ptr = WINPR_REINTERPRET_CAST(pSrcDst, INT16**, const INT16**);
-	rc = prims->yCbCrToRGB_16s8u_P3AC4R(ptr, 64 * 2, tile->data, tile->stride, progressive->format,
-	                                    &roi_64x64);
+	{
+		const INT16** ptr = WINPR_REINTERPRET_CAST(pSrcDst, INT16**, const INT16**);
+		rc = prims->yCbCrToRGB_16s8u_P3AC4R(ptr, 64 * 2, tile->data, tile->stride,
+		                                    progressive->format, &roi_64x64);
+	}
 fail:
 	BufferPool_Return(progressive->bufferPool, pBuffer);
 	return rc;
 }
 
-static INLINE INT16 progressive_rfx_srl_read(RFX_PROGRESSIVE_UPGRADE_STATE* WINPR_RESTRICT state,
+static inline INT16 progressive_rfx_srl_read(RFX_PROGRESSIVE_UPGRADE_STATE* WINPR_RESTRICT state,
                                              UINT32 numBits)
 {
 	WINPR_ASSERT(state);
@@ -1062,7 +1071,7 @@ static INLINE INT16 progressive_rfx_srl_read(RFX_PROGRESSIVE_UPGRADE_STATE* WINP
 	return (INT16)(sign ? -1 * (int)mag : (INT16)mag);
 }
 
-static INLINE int
+static inline int
 progressive_rfx_upgrade_state_finish(RFX_PROGRESSIVE_UPGRADE_STATE* WINPR_RESTRICT state)
 {
 	UINT32 pad = 0;
@@ -1090,7 +1099,7 @@ progressive_rfx_upgrade_state_finish(RFX_PROGRESSIVE_UPGRADE_STATE* WINPR_RESTRI
 	return 1;
 }
 
-static INLINE int progressive_rfx_upgrade_block(RFX_PROGRESSIVE_UPGRADE_STATE* WINPR_RESTRICT state,
+static inline int progressive_rfx_upgrade_block(RFX_PROGRESSIVE_UPGRADE_STATE* WINPR_RESTRICT state,
                                                 INT16* WINPR_RESTRICT buffer,
                                                 INT16* WINPR_RESTRICT sign, UINT32 length,
                                                 UINT32 shift, WINPR_ATTR_UNUSED UINT32 bitPos,
@@ -1151,7 +1160,7 @@ static INLINE int progressive_rfx_upgrade_block(RFX_PROGRESSIVE_UPGRADE_STATE* W
 	return 1;
 }
 
-static INLINE int progressive_rfx_upgrade_component(
+static inline int progressive_rfx_upgrade_component(
     PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
     const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT shift,
     const RFX_COMPONENT_CODEC_QUANT* WINPR_RESTRICT bitPos,
@@ -1249,7 +1258,7 @@ static INLINE int progressive_rfx_upgrade_component(
 	                                     TRUE);
 }
 
-static INLINE int
+static inline int
 progressive_decompress_tile_upgrade(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
                                     RFX_PROGRESSIVE_TILE* WINPR_RESTRICT tile,
                                     PROGRESSIVE_BLOCK_REGION* WINPR_RESTRICT region,
@@ -1417,15 +1426,17 @@ progressive_decompress_tile_upgrade(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progress
 	if (status < 0)
 		goto fail;
 
-	const INT16** ptr = WINPR_REINTERPRET_CAST(pSrcDst, INT16**, const INT16**);
-	status = prims->yCbCrToRGB_16s8u_P3AC4R(ptr, 64 * 2, tile->data, tile->stride,
-	                                        progressive->format, &roi_64x64);
+	{
+		const INT16** ptr = WINPR_REINTERPRET_CAST(pSrcDst, INT16**, const INT16**);
+		status = prims->yCbCrToRGB_16s8u_P3AC4R(ptr, 64 * 2, tile->data, tile->stride,
+		                                        progressive->format, &roi_64x64);
+	}
 fail:
 	BufferPool_Return(progressive->bufferPool, pBuffer);
 	return status;
 }
 
-static INLINE BOOL progressive_tile_read_upgrade(
+static inline BOOL progressive_tile_read_upgrade(
     PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive, wStream* WINPR_RESTRICT s, UINT16 blockType,
     UINT32 blockLen, PROGRESSIVE_SURFACE_CONTEXT* WINPR_RESTRICT surface,
     PROGRESSIVE_BLOCK_REGION* WINPR_RESTRICT region,
@@ -1503,11 +1514,12 @@ static INLINE BOOL progressive_tile_read_upgrade(
 	return progressive_surface_tile_replace(surface, region, &tile, TRUE);
 }
 
-static INLINE BOOL progressive_tile_read(
-    PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive, BOOL simple, wStream* WINPR_RESTRICT s,
-    UINT16 blockType, UINT32 blockLen, PROGRESSIVE_SURFACE_CONTEXT* WINPR_RESTRICT surface,
-    PROGRESSIVE_BLOCK_REGION* WINPR_RESTRICT region,
-    WINPR_ATTR_UNUSED const PROGRESSIVE_BLOCK_CONTEXT* WINPR_RESTRICT context)
+static inline BOOL
+progressive_tile_read(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive, BOOL simple,
+                      wStream* WINPR_RESTRICT s, UINT16 blockType, UINT32 blockLen,
+                      PROGRESSIVE_SURFACE_CONTEXT* WINPR_RESTRICT surface,
+                      PROGRESSIVE_BLOCK_REGION* WINPR_RESTRICT region,
+                      WINPR_ATTR_UNUSED const PROGRESSIVE_BLOCK_CONTEXT* WINPR_RESTRICT context)
 {
 	RFX_PROGRESSIVE_TILE tile = { 0 };
 	size_t expect = simple ? 16 : 17;
@@ -1593,11 +1605,12 @@ static void CALLBACK progressive_process_tiles_tile_work_callback(PTP_CALLBACK_I
 	}
 }
 
-static INLINE SSIZE_T progressive_process_tiles(
-    PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive, wStream* WINPR_RESTRICT s,
-    PROGRESSIVE_BLOCK_REGION* WINPR_RESTRICT region,
-    PROGRESSIVE_SURFACE_CONTEXT* WINPR_RESTRICT surface,
-    const PROGRESSIVE_BLOCK_CONTEXT* WINPR_RESTRICT context)
+static inline SSIZE_T
+progressive_process_tiles(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
+                          wStream* WINPR_RESTRICT s,
+                          PROGRESSIVE_BLOCK_REGION* WINPR_RESTRICT region,
+                          PROGRESSIVE_SURFACE_CONTEXT* WINPR_RESTRICT surface,
+                          const PROGRESSIVE_BLOCK_CONTEXT* WINPR_RESTRICT context)
 {
 	int status = 0;
 	size_t end = 0;
@@ -1628,8 +1641,8 @@ static INLINE SSIZE_T progressive_process_tiles(
 
 		if (blockLen < 6)
 		{
-			WLog_Print(progressive->log, WLOG_ERROR, "Expected >= %" PRIu32 " remaining %" PRIuz, 6,
-			           blockLen);
+			WLog_Print(progressive->log, WLOG_ERROR, "Expected >= %" PRIu32 " remaining %" PRIu32,
+			           6u, blockLen);
 			return -1003;
 		}
 		if (!Stream_CheckAndLogRequiredLength(TAG, s, blockLen - 6))
@@ -1698,9 +1711,8 @@ static INLINE SSIZE_T progressive_process_tiles(
 
 		if (progressive->rfx_context->priv->UseThreads)
 		{
-			progressive->work_objects[idx] =
-			    CreateThreadpoolWork(progressive_process_tiles_tile_work_callback, (void*)param,
-			                         &progressive->rfx_context->priv->ThreadPoolEnv);
+			progressive->work_objects[idx] = CreateThreadpoolWork(
+			    progressive_process_tiles_tile_work_callback, (void*)param, NULL);
 			if (!progressive->work_objects[idx])
 			{
 				WLog_Print(progressive->log, WLOG_ERROR,
@@ -1743,7 +1755,7 @@ fail:
 	return (SSIZE_T)(end - start);
 }
 
-static INLINE SSIZE_T progressive_wb_sync(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
+static inline SSIZE_T progressive_wb_sync(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
                                           wStream* WINPR_RESTRICT s, UINT16 blockType,
                                           UINT32 blockLen)
 {
@@ -1758,7 +1770,7 @@ static INLINE SSIZE_T progressive_wb_sync(PROGRESSIVE_CONTEXT* WINPR_RESTRICT pr
 	{
 		WLog_Print(progressive->log, WLOG_ERROR,
 		           "PROGRESSIVE_BLOCK_SYNC::blockLen = 0x%08" PRIx32 " != 0x%08" PRIx32,
-		           sync.blockLen, 12);
+		           sync.blockLen, 12u);
 		return -1005;
 	}
 
@@ -1795,7 +1807,7 @@ static INLINE SSIZE_T progressive_wb_sync(PROGRESSIVE_CONTEXT* WINPR_RESTRICT pr
 	return 0;
 }
 
-static INLINE SSIZE_T progressive_wb_frame_begin(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
+static inline SSIZE_T progressive_wb_frame_begin(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
                                                  wStream* WINPR_RESTRICT s, UINT16 blockType,
                                                  UINT32 blockLen)
 {
@@ -1808,7 +1820,7 @@ static INLINE SSIZE_T progressive_wb_frame_begin(PROGRESSIVE_CONTEXT* WINPR_REST
 	{
 		WLog_Print(progressive->log, WLOG_ERROR,
 		           " RFX_PROGRESSIVE_FRAME_BEGIN::blockLen = 0x%08" PRIx32 " != 0x%08" PRIx32,
-		           frameBegin.blockLen, 12);
+		           frameBegin.blockLen, 12u);
 		return -1005;
 	}
 
@@ -1847,7 +1859,7 @@ static INLINE SSIZE_T progressive_wb_frame_begin(PROGRESSIVE_CONTEXT* WINPR_REST
 	return 0;
 }
 
-static INLINE SSIZE_T progressive_wb_frame_end(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
+static inline SSIZE_T progressive_wb_frame_end(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
                                                wStream* WINPR_RESTRICT s, UINT16 blockType,
                                                UINT32 blockLen)
 {
@@ -1860,15 +1872,15 @@ static INLINE SSIZE_T progressive_wb_frame_end(PROGRESSIVE_CONTEXT* WINPR_RESTRI
 	{
 		WLog_Print(progressive->log, WLOG_ERROR,
 		           " RFX_PROGRESSIVE_FRAME_END::blockLen = 0x%08" PRIx32 " != 0x%08" PRIx32,
-		           frameEnd.blockLen, 6);
+		           frameEnd.blockLen, 6u);
 		return -1005;
 	}
 
 	if (Stream_GetRemainingLength(s) != 0)
 	{
 		WLog_Print(progressive->log, WLOG_ERROR,
-		           "ProgressiveFrameEnd short %" PRIuz ", expected %" PRIuz,
-		           Stream_GetRemainingLength(s), 0);
+		           "ProgressiveFrameEnd short %" PRIuz ", expected %u",
+		           Stream_GetRemainingLength(s), 0U);
 		return -1008;
 	}
 
@@ -1885,7 +1897,7 @@ static INLINE SSIZE_T progressive_wb_frame_end(PROGRESSIVE_CONTEXT* WINPR_RESTRI
 	return 0;
 }
 
-static INLINE SSIZE_T progressive_wb_context(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
+static inline SSIZE_T progressive_wb_context(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
                                              wStream* WINPR_RESTRICT s, UINT16 blockType,
                                              UINT32 blockLen)
 {
@@ -1896,8 +1908,8 @@ static INLINE SSIZE_T progressive_wb_context(PROGRESSIVE_CONTEXT* WINPR_RESTRICT
 	if (context->blockLen != 10)
 	{
 		WLog_Print(progressive->log, WLOG_ERROR,
-		           "RFX_PROGRESSIVE_CONTEXT::blockLen = 0x%08" PRIx32 " != 0x%08" PRIx32,
-		           context->blockLen, 10);
+		           "RFX_PROGRESSIVE_CONTEXT::blockLen = 0x%08" PRIx32 " != 0x%08x",
+		           context->blockLen, 10u);
 		return -1005;
 	}
 
@@ -1933,9 +1945,10 @@ static INLINE SSIZE_T progressive_wb_context(PROGRESSIVE_CONTEXT* WINPR_RESTRICT
 	return 0;
 }
 
-static INLINE SSIZE_T progressive_wb_read_region_header(
-    PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive, wStream* WINPR_RESTRICT s, UINT16 blockType,
-    UINT32 blockLen, PROGRESSIVE_BLOCK_REGION* WINPR_RESTRICT region)
+static inline SSIZE_T
+progressive_wb_read_region_header(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
+                                  wStream* WINPR_RESTRICT s, UINT16 blockType, UINT32 blockLen,
+                                  PROGRESSIVE_BLOCK_REGION* WINPR_RESTRICT region)
 {
 	region->usedTiles = 0;
 
@@ -1955,8 +1968,7 @@ static INLINE SSIZE_T progressive_wb_read_region_header(
 	if (region->tileSize != 64)
 	{
 		WLog_Print(progressive->log, WLOG_ERROR,
-		           "ProgressiveRegion tile size %" PRIu8 ", expected %" PRIuz, region->tileSize,
-		           64);
+		           "ProgressiveRegion tile size %" PRIu8 ", expected %u", region->tileSize, 64U);
 		return -1012;
 	}
 
@@ -1970,8 +1982,8 @@ static INLINE SSIZE_T progressive_wb_read_region_header(
 	if (region->numQuant > 7)
 	{
 		WLog_Print(progressive->log, WLOG_ERROR,
-		           "ProgressiveRegion quant count too high %" PRIu8 ", expected < %" PRIuz,
-		           region->numQuant, 7);
+		           "ProgressiveRegion quant count too high %" PRIu8 ", expected < %u",
+		           region->numQuant, 7U);
 		return -1014;
 	}
 
@@ -2021,7 +2033,7 @@ static INLINE SSIZE_T progressive_wb_read_region_header(
 	return rc;
 }
 
-static INLINE SSIZE_T progressive_wb_skip_region(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
+static inline SSIZE_T progressive_wb_skip_region(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
                                                  wStream* WINPR_RESTRICT s, UINT16 blockType,
                                                  UINT32 blockLen)
 {
@@ -2038,7 +2050,7 @@ static INLINE SSIZE_T progressive_wb_skip_region(PROGRESSIVE_CONTEXT* WINPR_REST
 	return rc;
 }
 
-static INLINE SSIZE_T progressive_wb_region(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
+static inline SSIZE_T progressive_wb_region(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
                                             wStream* WINPR_RESTRICT s, UINT16 blockType,
                                             UINT32 blockLen,
                                             PROGRESSIVE_SURFACE_CONTEXT* WINPR_RESTRICT surface,
@@ -2159,7 +2171,7 @@ static INLINE SSIZE_T progressive_wb_region(PROGRESSIVE_CONTEXT* WINPR_RESTRICT
 	return rc;
 }
 
-static INLINE SSIZE_T progressive_parse_block(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
+static inline SSIZE_T progressive_parse_block(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
                                               wStream* WINPR_RESTRICT s,
                                               PROGRESSIVE_SURFACE_CONTEXT* WINPR_RESTRICT surface,
                                               PROGRESSIVE_BLOCK_REGION* WINPR_RESTRICT region)
@@ -2228,7 +2240,7 @@ static INLINE SSIZE_T progressive_parse_block(PROGRESSIVE_CONTEXT* WINPR_RESTRIC
 	return rc;
 }
 
-static INLINE BOOL update_tiles(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
+static inline BOOL update_tiles(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
                                 PROGRESSIVE_SURFACE_CONTEXT* WINPR_RESTRICT surface,
                                 BYTE* WINPR_RESTRICT pDstData, UINT32 DstFormat, UINT32 nDstStep,
                                 UINT32 nXDst, UINT32 nYDst,
@@ -2279,18 +2291,19 @@ static INLINE BOOL update_tiles(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
 
 		for (UINT32 j = 0; j < nbUpdateRects; j++)
 		{
+			rc = FALSE;
 			const RECTANGLE_16* rect = &updateRects[j];
 			if (rect->left < updateRect.left)
-				goto fail;
+				break;
 			const UINT32 nXSrc = rect->left - updateRect.left;
 			const UINT32 nYSrc = rect->top - updateRect.top;
 			const UINT32 width = rect->right - rect->left;
 			const UINT32 height = rect->bottom - rect->top;
 
 			if (rect->left + width > surface->width)
-				goto fail;
+				break;
 			if (rect->top + height > surface->height)
-				goto fail;
+				break;
 			rc = freerdp_image_copy_no_overlap(
 			    pDstData, DstFormat, nDstStep, rect->left, rect->top, width, height, tile->data,
 			    progressive->format, tile->stride, nXSrc, nYSrc, NULL, FREERDP_KEEP_DST_ALPHA);
@@ -2302,6 +2315,8 @@ static INLINE BOOL update_tiles(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
 		}
 
 		region16_uninit(&updateRegion);
+		if (!rc)
+			goto fail;
 		tile->dirty = FALSE;
 	}
 
@@ -2362,14 +2377,16 @@ INT32 progressive_decompress(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
 			goto fail;
 	}
 
-	const size_t end = Stream_GetPosition(s);
-	if ((end - start) != SrcSize)
 	{
-		WLog_Print(progressive->log, WLOG_ERROR,
-		           "total block len %" PRIuz " does not match read data %" PRIu32, end - start,
-		           SrcSize);
-		rc = -1041;
-		goto fail;
+		const size_t end = Stream_GetPosition(s);
+		if ((end - start) != SrcSize)
+		{
+			WLog_Print(progressive->log, WLOG_ERROR,
+			           "total block len %" PRIuz " does not match read data %" PRIu32, end - start,
+			           SrcSize);
+			rc = -1041;
+			goto fail;
+		}
 	}
 
 	if (!update_tiles(progressive, surface, pDstData, DstFormat, nDstStep, nXDst, nYDst, region,
@@ -2379,8 +2396,9 @@ fail:
 	return rc;
 }
 
-BOOL progressive_rfx_write_message_progressive_simple(PROGRESSIVE_CONTEXT* progressive, wStream* s,
-                                                      const RFX_MESSAGE* msg)
+BOOL progressive_rfx_write_message_progressive_simple(
+    PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive, wStream* WINPR_RESTRICT s,
+    const RFX_MESSAGE* WINPR_RESTRICT msg)
 {
 	RFX_CONTEXT* context = NULL;
 
@@ -2478,7 +2496,7 @@ int progressive_compress(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
 			r->width = MIN(64, WINPR_ASSERTING_INT_CAST(UINT16, Width - x));
 			r->height = MIN(64, WINPR_ASSERTING_INT_CAST(UINT16, Height - y));
 
-			if (x + 64 >= Width)
+			if (x + 64UL >= Width)
 			{
 				y += 64;
 				x = 0;
@@ -2513,9 +2531,11 @@ int progressive_compress(PROGRESSIVE_CONTEXT* WINPR_RESTRICT progressive,
 	if (!rc)
 		goto fail;
 
-	const size_t pos = Stream_GetPosition(s);
-	WINPR_ASSERT(pos <= UINT32_MAX);
-	*pDstSize = (UINT32)pos;
+	{
+		const size_t pos = Stream_GetPosition(s);
+		WINPR_ASSERT(pos <= UINT32_MAX);
+		*pDstSize = (UINT32)pos;
+	}
 	*ppDstData = Stream_Buffer(s);
 	res = 1;
 fail:
