@@ -121,6 +121,9 @@ static int freerdp_connect_begin(freerdp* instance)
 	if (status)
 		status = freerdp_settings_enforce_monitor_exists(settings);
 
+	if (status)
+		status = freerdp_settings_enforce_consistency(settings);
+
 	if (status)
 		status = freerdp_settings_check_client_after_preconnect(settings);
 
@@ -299,6 +302,7 @@ freerdp_connect_finally:
 	return status;
 }
 
+#if !defined(WITHOUT_FREERDP_3x_DEPRECATED)
 BOOL freerdp_abort_connect(freerdp* instance)
 {
 	if (!instance)
@@ -306,6 +310,7 @@ BOOL freerdp_abort_connect(freerdp* instance)
 
 	return freerdp_abort_connect_context(instance->context);
 }
+#endif
 
 BOOL freerdp_abort_connect_context(rdpContext* context)
 {
@@ -313,7 +318,6 @@ BOOL freerdp_abort_connect_context(rdpContext* context)
 		return FALSE;
 
 	freerdp_set_last_error_if_not(context, FREERDP_ERROR_CONNECT_CANCELLED);
-
 	return utils_abort_connect(context->rdp);
 }
 
@@ -375,16 +379,16 @@ DWORD freerdp_get_event_handles(rdpContext* context, HANDLE* events, DWORD count
 	WINPR_ASSERT(context->rdp);
 	WINPR_ASSERT(events || (count == 0));
 
-	nCount += transport_get_event_handles(context->rdp->transport, events, count);
-
-	if (nCount == 0)
+	const size_t rrc = rdp_get_event_handles(context->rdp, &events[nCount], count - nCount);
+	if (rrc == 0)
 		return 0;
 
+	nCount += WINPR_ASSERTING_INT_CAST(uint32_t, rrc);
+
 	if (events && (nCount < count + 2))
 	{
 		events[nCount++] = freerdp_channels_get_event_handle(context->instance);
 		events[nCount++] = getChannelErrorEventHandle(context);
-		events[nCount++] = utils_get_abort_event(context->rdp);
 	}
 	else
 		return 0;
@@ -593,29 +597,45 @@ static BOOL freerdp_send_channel_packet(freerdp* instance, UINT16 channelId, siz
 BOOL freerdp_disconnect(freerdp* instance)
 {
 	BOOL rc = TRUE;
-	rdpRdp* rdp = NULL;
-	rdp_update_internal* up = NULL;
 
 	if (!instance || !instance->context)
 		return FALSE;
 
-	rdp = instance->context->rdp;
+	rdpRdp* rdp = instance->context->rdp;
+	if (rdp)
+	{
+		/* Try to send a [MS-RDPBCGR] 1.3.1.4.1 User-Initiated on Client PDU, we don't care about
+		 * success */
+		if (freerdp_get_last_error(instance->context) == FREERDP_ERROR_CONNECT_CANCELLED)
+		{
+			(void)mcs_send_disconnect_provider_ultimatum(rdp->mcs,
+			                                             Disconnect_Ultimatum_user_requested);
+		}
+	}
+
 	utils_abort_connect(rdp);
 
 	if (!rdp_client_disconnect(rdp))
 		rc = FALSE;
 
-	up = update_cast(rdp->update);
+	rdp_update_internal* up = NULL;
+	if (rdp && rdp->update)
+	{
+		up = update_cast(rdp->update);
 
-	update_post_disconnect(rdp->update);
+		update_post_disconnect(rdp->update);
+	}
 
 	IFCALL(instance->PostDisconnect, instance);
 
-	if (up->pcap_rfx)
+	if (up)
 	{
-		up->dump_rfx = FALSE;
-		pcap_close(up->pcap_rfx);
-		up->pcap_rfx = NULL;
+		if (up->pcap_rfx)
+		{
+			up->dump_rfx = FALSE;
+			pcap_close(up->pcap_rfx);
+			up->pcap_rfx = NULL;
+		}
 	}
 
 	freerdp_channels_close(instance->context->channels, instance);
@@ -626,11 +646,13 @@ BOOL freerdp_disconnect(freerdp* instance)
 	return rc;
 }
 
+#if !defined(WITHOUT_FREERDP_3x_DEPRECATED)
 BOOL freerdp_disconnect_before_reconnect(freerdp* instance)
 {
 	WINPR_ASSERT(instance);
 	return freerdp_disconnect_before_reconnect_context(instance->context);
 }
+#endif
 
 BOOL freerdp_disconnect_before_reconnect_context(rdpContext* context)
 {
@@ -659,13 +681,15 @@ BOOL freerdp_reconnect(freerdp* instance)
 	return rdp_client_reconnect(rdp);
 }
 
-BOOL freerdp_shall_disconnect(freerdp* instance)
+#if !defined(WITHOUT_FREERDP_3x_DEPRECATED)
+BOOL freerdp_shall_disconnect(const freerdp* instance)
 {
 	if (!instance)
 		return FALSE;
 
 	return freerdp_shall_disconnect_context(instance->context);
 }
+#endif
 
 BOOL freerdp_shall_disconnect_context(const rdpContext* context)
 {
@@ -770,6 +794,59 @@ BOOL freerdp_context_new(freerdp* instance)
 	return freerdp_context_new_ex(instance, NULL);
 }
 
+static BOOL freerdp_common_context(rdpContext* context, AccessTokenType tokenType, char** token,
+                                   size_t count, ...)
+{
+	BOOL rc = FALSE;
+
+	WINPR_ASSERT(context);
+	if (!context->instance || !context->instance->GetAccessToken)
+		return TRUE;
+
+	va_list ap;
+	va_start(ap, count);
+	switch (tokenType)
+	{
+		case ACCESS_TOKEN_TYPE_AAD:
+			if (count != 2)
+			{
+				WLog_ERR(TAG,
+				         "ACCESS_TOKEN_TYPE_AAD expected 2 additional arguments, but got %" PRIuz
+				         ", aborting",
+				         count);
+			}
+			else
+			{
+				const char* scope = va_arg(ap, const char*);
+				const char* req_cnf = va_arg(ap, const char*);
+				rc = context->instance->GetAccessToken(context->instance, tokenType, token, count,
+				                                       scope, req_cnf);
+			}
+			break;
+		case ACCESS_TOKEN_TYPE_AVD:
+			if (count != 0)
+			{
+				WLog_WARN(TAG,
+				          "ACCESS_TOKEN_TYPE_AVD expected 0 additional arguments, but got %" PRIuz
+				          ", ignoring",
+				          count);
+			}
+			else
+			{
+				rc = context->instance->GetAccessToken(context->instance, tokenType, token, count);
+			}
+			break;
+		default:
+			break;
+	}
+	va_end(ap);
+
+	if (!rc)
+		freerdp_set_last_error_if_not(context, FREERDP_ERROR_CONNECT_ACCESS_DENIED);
+
+	return rc;
+}
+
 BOOL freerdp_context_new_ex(freerdp* instance, rdpSettings* settings)
 {
 	rdpRdp* rdp = NULL;
@@ -850,10 +927,19 @@ BOOL freerdp_context_new_ex(freerdp* instance, rdpSettings* settings)
 	if (!context->dump)
 		goto fail;
 
+	/* Fallback:
+	 * Client common library might set a function pointer to handle this, but here we provide a
+	 * default implementation that simply calls instance->GetAccessToken.
+	 */
+	if (!freerdp_set_common_access_token(context, freerdp_common_context))
+		goto fail;
+
 	IFCALLRET(instance->ContextNew, ret, instance, context);
 
-	if (ret)
-		return TRUE;
+	if (!ret)
+		goto fail;
+
+	return TRUE;
 
 fail:
 	freerdp_context_free(instance);
@@ -934,13 +1020,13 @@ void freerdp_context_free(freerdp* instance)
 	instance->heartbeat = NULL; /* owned by rdpRdp */
 }
 
-int freerdp_get_disconnect_ultimatum(rdpContext* context)
+int freerdp_get_disconnect_ultimatum(const rdpContext* context)
 {
 	WINPR_ASSERT(context);
 	return context->disconnectUltimatum;
 }
 
-UINT32 freerdp_error_info(freerdp* instance)
+UINT32 freerdp_error_info(const freerdp* instance)
 {
 	WINPR_ASSERT(instance);
 	WINPR_ASSERT(instance->context);
@@ -964,7 +1050,7 @@ BOOL freerdp_send_error_info(rdpRdp* rdp)
 	return rdp_send_error_info(rdp);
 }
 
-UINT32 freerdp_get_last_error(rdpContext* context)
+UINT32 freerdp_get_last_error(const rdpContext* context)
 {
 	WINPR_ASSERT(context);
 	return context->LastError;
@@ -1064,26 +1150,26 @@ void freerdp_set_last_error_ex(rdpContext* context, UINT32 lastError, const char
 	{
 		if (WLog_IsLevelActive(context->log, WLOG_ERROR))
 		{
-			WLog_PrintMessage(context->log, WLOG_MESSAGE_TEXT, WLOG_ERROR, (size_t)line, file, fkt,
-			                  "%s [0x%08" PRIX32 "]", freerdp_get_last_error_name(lastError),
-			                  lastError);
+			WLog_PrintTextMessage(context->log, WLOG_ERROR, (size_t)line, file, fkt,
+			                      "%s [0x%08" PRIX32 "]", freerdp_get_last_error_name(lastError),
+			                      lastError);
 		}
 	}
 
 	if (lastError == FREERDP_ERROR_SUCCESS)
 	{
 		if (WLog_IsLevelActive(context->log, WLOG_DEBUG))
-			WLog_PrintMessage(context->log, WLOG_MESSAGE_TEXT, WLOG_DEBUG, (size_t)line, file, fkt,
-			                  "resetting error state");
+			WLog_PrintTextMessage(context->log, WLOG_DEBUG, (size_t)line, file, fkt,
+			                      "resetting error state");
 	}
 	else if (context->LastError != FREERDP_ERROR_SUCCESS)
 	{
 		if (WLog_IsLevelActive(context->log, WLOG_ERROR))
 		{
-			WLog_PrintMessage(context->log, WLOG_MESSAGE_TEXT, WLOG_ERROR, (size_t)line, file, fkt,
-			                  "TODO: Trying to set error code %s, but %s already set!",
-			                  freerdp_get_last_error_name(lastError),
-			                  freerdp_get_last_error_name(context->LastError));
+			WLog_PrintTextMessage(context->log, WLOG_ERROR, (size_t)line, file, fkt,
+			                      "TODO: Trying to set error code %s, but %s already set!",
+			                      freerdp_get_last_error_name(lastError),
+			                      freerdp_get_last_error_name(context->LastError));
 		}
 	}
 	context->LastError = lastError;
@@ -1178,11 +1264,12 @@ void freerdp_free(freerdp* instance)
 	free(instance);
 }
 
-ULONG freerdp_get_transport_sent(rdpContext* context, BOOL resetCount)
+ULONG freerdp_get_transport_sent(const rdpContext* context, BOOL resetCount)
 {
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(context->rdp);
-	return transport_get_bytes_sent(context->rdp->transport, resetCount);
+	UINT64 rc = transport_get_bytes_sent(context->rdp->transport, resetCount);
+	return WINPR_CXX_COMPAT_CAST(ULONG, MIN(rc, UINT32_MAX));
 }
 
 BOOL freerdp_nla_impersonate(rdpContext* context)
@@ -1219,14 +1306,13 @@ BOOL freerdp_nla_revert_to_self(rdpContext* context)
 	return nla_revert_to_self(nla);
 }
 
-UINT32 freerdp_get_nla_sspi_error(rdpContext* context)
+UINT32 freerdp_get_nla_sspi_error(const rdpContext* context)
 {
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(context->rdp);
 	WINPR_ASSERT(context->rdp->transport);
 
 	rdpNla* nla = transport_get_nla(context->rdp->transport);
-
 	return (UINT32)nla_get_sspi_error(nla);
 }
 
@@ -1281,13 +1367,13 @@ BOOL checkChannelErrorEvent(rdpContext* context)
  *
  * @return 0 on success, otherwise a Win32 error code
  */
-UINT getChannelError(rdpContext* context)
+UINT getChannelError(const rdpContext* context)
 {
 	WINPR_ASSERT(context);
 	return context->channelErrorNum;
 }
 
-const char* getChannelErrorDescription(rdpContext* context)
+const char* getChannelErrorDescription(const rdpContext* context)
 {
 	WINPR_ASSERT(context);
 	return context->errorDescription;
@@ -1315,7 +1401,7 @@ void setChannelError(rdpContext* context, UINT errorNum, WINPR_FORMAT_ARG const
 	(void)SetEvent(context->channelErrorEvent);
 }
 
-const char* freerdp_nego_get_routing_token(rdpContext* context, DWORD* length)
+const char* freerdp_nego_get_routing_token(const rdpContext* context, DWORD* length)
 {
 	if (!context || !context->rdp)
 		return NULL;
@@ -1387,22 +1473,18 @@ static void test_mcs_free(rdpMcs* mcs)
 	if (!mcs)
 		return;
 
-	rdpTransport* transport = mcs->transport;
-	rdpContext* context = transport_get_context(transport);
-	if (context)
+	if (mcs->context)
 	{
-		rdpSettings* settings = context->settings;
+		rdpSettings* settings = mcs->context->settings;
 		freerdp_settings_free(settings);
 	}
-	free(context);
-	transport_free(transport);
+	free(mcs->context);
 
 	mcs_free(mcs);
 }
 
 static rdpMcs* test_mcs_new(void)
 {
-	rdpTransport* transport = NULL;
 	rdpSettings* settings = freerdp_settings_new(0);
 	rdpContext* context = calloc(1, sizeof(rdpContext));
 
@@ -1414,13 +1496,9 @@ static rdpMcs* test_mcs_new(void)
 	if (!context)
 		goto fail;
 	context->settings = settings;
-	transport = transport_new(context);
-	if (!transport)
-		goto fail;
-	return mcs_new(transport);
+	return mcs_new(context);
 
 fail:
-	transport_free(transport);
 	free(context);
 	freerdp_settings_free(settings);
 
@@ -1468,3 +1546,38 @@ BOOL freerdp_persist_credentials(rdpContext* context)
 	WINPR_ASSERT(context->rdp);
 	return utils_persist_credentials(context->rdp->originalSettings, context->rdp->settings);
 }
+
+const char* freerdp_disconnect_reason_string(int reason)
+{
+	switch (reason)
+	{
+		case Disconnect_Ultimatum_domain_disconnected:
+			return "rn-domain-disconnected";
+		case Disconnect_Ultimatum_provider_initiated:
+			return "rn-provider-initiated";
+		case Disconnect_Ultimatum_token_purged:
+			return "rn-token-purged";
+		case Disconnect_Ultimatum_user_requested:
+			return "rn-user-requested";
+		case Disconnect_Ultimatum_channel_purged:
+			return "rn-channel-purged";
+		default:
+			return "rn-unknown";
+	}
+}
+
+BOOL freerdp_set_common_access_token(rdpContext* context,
+                                     pGetCommonAccessToken GetCommonAccessToken)
+{
+	WINPR_ASSERT(context);
+	WINPR_ASSERT(context->rdp);
+	context->rdp->GetCommonAccessToken = GetCommonAccessToken;
+	return TRUE;
+}
+
+pGetCommonAccessToken freerdp_get_common_access_token(const rdpContext* context)
+{
+	WINPR_ASSERT(context);
+	WINPR_ASSERT(context->rdp);
+	return context->rdp->GetCommonAccessToken;
+}
