@@ -201,14 +201,16 @@ BYTE* winpr_bitmap_construct_header(size_t width, size_t height, size_t bpp)
 	BYTE* result = NULL;
 	WINPR_BITMAP_FILE_HEADER bf = { 0 };
 	WINPR_BITMAP_INFO_HEADER bi = { 0 };
-	wStream* s = NULL;
-	size_t imgSize = 0;
 
-	imgSize = width * height * (bpp / 8);
+	size_t stride = (width * bpp + 7) / 8;
+	if ((stride % 4) != 0)
+		stride += 4 - (stride % 4);
+
+	size_t imgSize = stride * height;
 	if ((width > INT32_MAX) || (height > INT32_MAX) || (bpp > UINT16_MAX) || (imgSize > UINT32_MAX))
 		return NULL;
 
-	s = Stream_New(NULL, WINPR_IMAGE_BMP_HEADER_LEN);
+	wStream* s = Stream_New(NULL, WINPR_IMAGE_BMP_HEADER_LEN);
 	if (!s)
 		return NULL;
 
@@ -277,7 +279,10 @@ static void* winpr_bitmap_write_buffer(const BYTE* data, WINPR_ATTR_UNUSED size_
 	WINPR_ASSERT(data || (size == 0));
 
 	void* result = NULL;
-	const size_t bpp_stride = 1ull * width * (bpp / 8);
+	size_t bpp_stride = 1ull * width * (bpp / 8);
+	if ((bpp_stride % 4) != 0)
+		bpp_stride += 4 - (bpp_stride % 4);
+
 	if (bpp_stride > UINT32_MAX)
 		return NULL;
 
@@ -293,7 +298,7 @@ static void* winpr_bitmap_write_buffer(const BYTE* data, WINPR_ATTR_UNUSED size_
 		goto fail;
 	Stream_Write(s, bmp_header, WINPR_IMAGE_BMP_HEADER_LEN);
 
-	if (!Stream_EnsureRemainingCapacity(s, 1ULL * stride * height))
+	if (!Stream_EnsureRemainingCapacity(s, 1ULL * bpp_stride * height))
 		goto fail;
 
 	for (size_t y = 0; y < height; y++)
@@ -301,13 +306,16 @@ static void* winpr_bitmap_write_buffer(const BYTE* data, WINPR_ATTR_UNUSED size_
 		const BYTE* line = &data[stride * y];
 
 		Stream_Write(s, line, stride);
+		Stream_Zero(s, bpp_stride - stride);
 	}
 
 	result = Stream_Buffer(s);
-	const size_t pos = Stream_GetPosition(s);
-	if (pos > UINT32_MAX)
-		goto fail;
-	*pSize = (UINT32)pos;
+	{
+		const size_t pos = Stream_GetPosition(s);
+		if (pos > UINT32_MAX)
+			goto fail;
+		*pSize = (UINT32)pos;
+	}
 fail:
 	Stream_Free(s, result == NULL);
 	free(bmp_header);
@@ -335,22 +343,26 @@ int winpr_bitmap_write_ex(const char* filename, const BYTE* data, size_t stride,
 	if (stride == 0)
 		stride = bpp_stride;
 
-	UINT32 bmpsize = 0;
-	const size_t size = stride * 1ull * height;
-	bmpdata = winpr_bitmap_write_buffer(data, size, (UINT32)width, (UINT32)height, (UINT32)stride,
-	                                    (UINT32)bpp, &bmpsize);
-	if (!bmpdata)
-		goto fail;
-
-	fp = winpr_fopen(filename, "w+b");
-	if (!fp)
 	{
-		WLog_ERR(TAG, "failed to open file %s", filename);
-		goto fail;
-	}
+		UINT32 bmpsize = 0;
+		{
+			const size_t size = stride * 1ull * height;
+			bmpdata = winpr_bitmap_write_buffer(data, size, (UINT32)width, (UINT32)height,
+			                                    (UINT32)stride, (UINT32)bpp, &bmpsize);
+		}
+		if (!bmpdata)
+			goto fail;
 
-	if (fwrite(bmpdata, bmpsize, 1, fp) != 1)
-		goto fail;
+		fp = winpr_fopen(filename, "w+b");
+		if (!fp)
+		{
+			WLog_ERR(TAG, "failed to open file %s", filename);
+			goto fail;
+		}
+
+		if (fwrite(bmpdata, bmpsize, 1, fp) != 1)
+			goto fail;
+	}
 
 	ret = 0;
 fail:
@@ -366,14 +378,17 @@ static int write_and_free(const char* filename, void* data, size_t size)
 	if (!data)
 		goto fail;
 
-	FILE* fp = winpr_fopen(filename, "w+b");
-	if (!fp)
-		goto fail;
+	{
+		FILE* fp = winpr_fopen(filename, "w+b");
+		if (!fp)
+			goto fail;
 
-	size_t w = fwrite(data, 1, size, fp);
-	(void)fclose(fp);
-
-	status = (w == size) ? 1 : -1;
+		{
+			const size_t w = fwrite(data, 1, size, fp);
+			(void)fclose(fp);
+			status = (w == size) ? 1 : -1;
+		}
+	}
 fail:
 	free(data);
 	return status;
@@ -420,14 +435,15 @@ static int winpr_image_bitmap_read_buffer(wImage* image, const BYTE* buffer, siz
 
 	image->type = WINPR_IMAGE_BITMAP;
 
-	const size_t pos = Stream_GetPosition(s);
-	const size_t expect = bf.bfOffBits;
-
-	if (pos != expect)
 	{
-		WLog_WARN(TAG, "pos=%" PRIuz ", expected %" PRIuz ", offset=" PRIuz, pos, expect,
-		          bmpoffset);
-		goto fail;
+		const size_t pos = Stream_GetPosition(s);
+		const size_t expect = bf.bfOffBits;
+		if (pos != expect)
+		{
+			WLog_WARN(TAG, "pos=%" PRIuz ", expected %" PRIuz ", offset=" PRIuz, pos, expect,
+			          bmpoffset);
+			goto fail;
+		}
 	}
 
 	if (!Stream_CheckAndLogRequiredCapacity(TAG, s, bi.biSizeImage))
@@ -459,33 +475,65 @@ static int winpr_image_bitmap_read_buffer(wImage* image, const BYTE* buffer, siz
 	}
 
 	image->bitsPerPixel = bi.biBitCount;
-	image->bytesPerPixel = (image->bitsPerPixel / 8UL);
-	const size_t bpp = (bi.biBitCount + 7UL) / 8UL;
-	image->scanline =
-	    WINPR_ASSERTING_INT_CAST(uint32_t, bi.biWidth) * WINPR_ASSERTING_INT_CAST(uint32_t, bpp);
-	const size_t bmpsize = 1ULL * image->scanline * image->height;
-	if (bmpsize != bi.biSizeImage)
-		WLog_WARN(TAG, "bmpsize=%" PRIuz " != bi.biSizeImage=%" PRIu32, bmpsize, bi.biSizeImage);
-	if (bi.biSizeImage < bmpsize)
-		goto fail;
-
-	image->data = NULL;
-	if (bi.biSizeImage > 0)
-		image->data = (BYTE*)malloc(bi.biSizeImage);
-
-	if (!image->data)
-		goto fail;
-
-	if (!vFlip)
-		Stream_Read(s, image->data, bi.biSizeImage);
-	else
 	{
-		BYTE* pDstData = &(image->data[(image->height - 1ull) * image->scanline]);
+		const size_t bpp = (bi.biBitCount + 7UL) / 8UL;
+		image->bytesPerPixel = WINPR_ASSERTING_INT_CAST(uint32_t, bpp);
+
+		image->scanline = WINPR_ASSERTING_INT_CAST(uint32_t, bi.biWidth) * image->bytesPerPixel;
+		if ((image->scanline % 4) != 0)
+			image->scanline += 4 - image->scanline % 4;
 
-		for (size_t index = 0; index < image->height; index++)
 		{
-			Stream_Read(s, pDstData, image->scanline);
-			pDstData -= image->scanline;
+			const size_t bmpsize = 1ULL * image->scanline * image->height;
+			if (bmpsize != bi.biSizeImage)
+				WLog_WARN(TAG, "bmpsize=%" PRIuz " != bi.biSizeImage=%" PRIu32, bmpsize,
+				          bi.biSizeImage);
+
+			{
+				size_t scanline = image->scanline;
+				if (bi.biSizeImage < bmpsize)
+				{
+					/* Workaround for unaligned bitmaps */
+					const size_t uscanline = image->width * bpp;
+					const size_t unaligned = image->height * uscanline;
+					if (bi.biSizeImage != unaligned)
+						goto fail;
+					scanline = uscanline;
+				}
+
+				image->data = NULL;
+				{
+					const size_t asize = 1ULL * image->scanline * image->height;
+					if (asize > 0)
+						image->data = (BYTE*)malloc(asize);
+				}
+
+				if (!image->data)
+					goto fail;
+
+				if (!vFlip)
+				{
+					BYTE* pDstData = image->data;
+
+					for (size_t index = 0; index < image->height; index++)
+					{
+						Stream_Read(s, pDstData, scanline);
+						Stream_Seek(s, image->scanline - scanline);
+						pDstData += scanline;
+					}
+				}
+				else
+				{
+					BYTE* pDstData = &(image->data[(image->height - 1ull) * image->scanline]);
+
+					for (size_t index = 0; index < image->height; index++)
+					{
+						Stream_Read(s, pDstData, scanline);
+						Stream_Seek(s, image->scanline - scanline);
+						pDstData -= scanline;
+					}
+				}
+			}
 		}
 	}
 
@@ -624,6 +672,7 @@ static void* winpr_convert_to_jpeg(WINPR_ATTR_UNUSED const void* data,
 	*pSize = 0;
 
 #if !defined(WINPR_UTILS_IMAGE_JPEG)
+	WLog_WARN(TAG, "JPEG not supported in this build");
 	return NULL;
 #else
 	BYTE* outbuffer = NULL;
@@ -633,7 +682,7 @@ static void* winpr_convert_to_jpeg(WINPR_ATTR_UNUSED const void* data,
 	const size_t expect1 = 1ull * stride * height;
 	const size_t bytes = (bpp + 7) / 8;
 	const size_t expect2 = 1ull * width * height * bytes;
-	if (expect1 != expect2)
+	if (expect1 < expect2)
 		return NULL;
 	if (expect1 > size)
 		return NULL;
@@ -697,6 +746,7 @@ SSIZE_T winpr_convert_from_jpeg(WINPR_ATTR_UNUSED const BYTE* comp_data,
 	WINPR_ASSERT(ppdecomp_data);
 
 #if !defined(WINPR_UTILS_IMAGE_JPEG)
+	WLog_WARN(TAG, "JPEG not supported in this build");
 	return -1;
 #else
 	struct jpeg_decompress_struct cinfo = { 0 };
@@ -759,6 +809,7 @@ static void* winpr_convert_to_webp(WINPR_ATTR_UNUSED const void* data,
 	*pSize = 0;
 
 #if !defined(WINPR_UTILS_IMAGE_WEBP)
+	WLog_WARN(TAG, "WEBP not supported in this build");
 	return NULL;
 #else
 	size_t dstSize = 0;
@@ -806,6 +857,7 @@ SSIZE_T winpr_convert_from_webp(WINPR_ATTR_UNUSED const BYTE* comp_data,
 	*bpp = 0;
 	*ppdecomp_data = NULL;
 #if !defined(WINPR_UTILS_IMAGE_WEBP)
+	WLog_WARN(TAG, "WEBP not supported in this build");
 	return -1;
 #else
 
@@ -863,8 +915,8 @@ static void png_flush(WINPR_ATTR_UNUSED png_structp png_ptr)
 {
 }
 
-static SSIZE_T save_png_to_buffer(UINT32 bpp, UINT32 width, UINT32 height, const uint8_t* data,
-                                  size_t size, void** pDstData)
+static SSIZE_T save_png_to_buffer(UINT32 bpp, UINT32 width, uint32_t stride, UINT32 height,
+                                  const uint8_t* data, size_t size, void** pDstData)
 {
 	SSIZE_T rc = -1;
 	png_structp png_ptr = NULL;
@@ -879,9 +931,7 @@ static SSIZE_T save_png_to_buffer(UINT32 bpp, UINT32 width, UINT32 height, const
 
 	WINPR_ASSERT(pDstData);
 
-	const size_t bytes_per_pixel = (bpp + 7) / 8;
-	const size_t bytes_per_row = width * bytes_per_pixel;
-	if (size < bytes_per_row * height)
+	if (size < (1ULL * stride * height))
 		goto fail;
 
 	/* Initialize the write struct. */
@@ -914,18 +964,19 @@ static SSIZE_T save_png_to_buffer(UINT32 bpp, UINT32 width, UINT32 height, const
 	row_pointers = (png_byte**)png_malloc(png_ptr, height * sizeof(png_byte*));
 	for (size_t y = 0; y < height; ++y)
 	{
-		uint8_t* row = png_malloc(png_ptr, sizeof(uint8_t) * bytes_per_row);
+		const uint8_t* line = &data[y * stride];
+		uint8_t* row = png_malloc(png_ptr, sizeof(uint8_t) * stride);
 		row_pointers[y] = (png_byte*)row;
 		for (size_t x = 0; x < width; ++x)
 		{
 
-			*row++ = *data++;
+			*row++ = *line++;
 			if (bpp > 8)
-				*row++ = *data++;
+				*row++ = *line++;
 			if (bpp > 16)
-				*row++ = *data++;
+				*row++ = *line++;
 			if (bpp > 24)
-				*row++ = *data++;
+				*row++ = *line++;
 		}
 	}
 
@@ -1053,7 +1104,7 @@ static void* winpr_convert_to_png(WINPR_ATTR_UNUSED const void* data, WINPR_ATTR
 
 #if defined(WINPR_UTILS_IMAGE_PNG)
 	void* dst = NULL;
-	SSIZE_T rc = save_png_to_buffer(bpp, width, height, data, size, &dst);
+	SSIZE_T rc = save_png_to_buffer(bpp, width, stride, height, data, size, &dst);
 	if (rc <= 0)
 		return NULL;
 	*pSize = (UINT32)rc;
@@ -1081,6 +1132,7 @@ static void* winpr_convert_to_png(WINPR_ATTR_UNUSED const void* data, WINPR_ATTR
 		return dst;
 	}
 #else
+	WLog_WARN(TAG, "PNG not supported in this build");
 	return NULL;
 #endif
 }
@@ -1103,6 +1155,7 @@ SSIZE_T winpr_convert_from_png(WINPR_ATTR_UNUSED const BYTE* comp_data,
 	return lodepng_decode32((unsigned char**)ppdecomp_data, width, height, comp_data,
 	                        comp_data_bytes);
 #else
+	WLog_WARN(TAG, "PNG not supported in this build");
 	return -1;
 #endif
 }
